require 'forwardable'

module Axl

  # ==========
  # EXCEPTIONS
  # ==========

  class ConfigurationFileNotFoundError < StandardError

    def initialize(path)
    msg = "The configuration file could not be found at #{path}."
      super msg
    end

  end

  # ===================
  # CONFIGURATION CLASS
  # ===================

  class Configuration

    # =============
    # CLASS METHODS
    # =============

    class << self

      def load(path, env: 'production')
        Configuration.new(path, env: env)
      end

    end

    # ===========
    # INITIALIZER
    # ===========

    def initialize(path, env: nil)
      begin
        @config = YAML.load_file(path)[env.to_s]
      rescue Errno::ENOENT
        raise ConfigurationFileNotFoundError.new(path) unless Pathname.new(path).exist?
      end
    end

    # ======
    # PUBLIC
    # ======

    def to_h
      config.dup
    end

    # =======
    # PRIVATE
    # =======

    private

    def config
      @config
    end

    def method_missing(name, *args, &block) # :nodoc:
      if config.has_key? name.to_s
        config[name.to_s]
      else
        super name, *args, &block
      end
    end

  end

end
